@@ -147,7 +147,7 @@ char *do_browser(char *path)
 		/* If we selected the same filename as last time, fake a
 		 * press of the Enter key so that the file is read in. */
 		if (old_selected == selected)
-		    unget_kbinput(sc_seq_or(do_enter, 0), FALSE);
+		    unget_kbinput(KEY_ENTER, FALSE);
 	    }
 
 	    continue;
@@ -230,17 +230,11 @@ char *do_browser(char *path)
 			/* TRANSLATORS: This is a prompt. */
 			browser_refresh, _("Go To Directory"));
 
-	    /* If the directory begins with a newline (i.e. an
-	     * encoded null), treat it as though it's blank. */
-	    if (i < 0 || *answer == '\n') {
+	    if (i < 0) {
 		statusbar(_("Cancelled"));
 		continue;
 	    }
 
-	    /* Convert newlines to nulls in the directory name. */
-	    sunder(answer);
-	    align(&answer);
-
 	    path = free_and_assign(path, real_dir_from_tilde(answer));
 
 	    /* If the given path is relative, join it with the current path. */
@@ -379,8 +373,7 @@ char *do_browse_from(const char *inpath)
 		beep();
 		napms(1200);
 		return NULL;
-	    } else
-		align(&path);
+	    }
 	}
     }
 
@@ -714,7 +707,7 @@ void findnextfile(const char *needle)
     /* Save the settings of all flags. */
     memcpy(stash, flags, sizeof(flags));
 
-    /* Search forward, case insensitive and without regexes. */
+    /* Search forward, case insensitive, and without regexes. */
     UNSET(BACKWARDS_SEARCH);
     UNSET(CASE_SENSITIVE);
     UNSET(USE_REGEXP);
